//=============================================================================
// DawnsTriggerLight.
// A lightsource which can be triggered on or off and uses a Sprite to draw a
// corona that fades on and off with the light.
// Works in all modes of play, including NetMode
//
// Version 1.0
// By Dawn Knight
// 07/24/04
//
//=============================================================================
class DawnsTriggerLight extends Light;

//-----------------------------------------------------------------------------
// Variables.

var() float ChangeTime;        // Time light takes to change from on to off.
var() bool  bInitiallyOn;      // Whether it's initially on.
var() bool  bDelayFullOn;      // Delay then go full-on.
var() float RemainOnTime;      // How long the TriggerPound effect lasts

var   float InitialBrightness; // Initial brightness.
var   float Alpha, Direction;
var   actor SavedTrigger;
var   float poundTime;

var(DawnsTriggerLightSounds) sound TurnOnSound;
var(DawnsTriggerLightSounds) sound TurnOffSound;
var(DawnsTriggerLightSounds) sound RunningAmbientSound;

//-----------------------------------------------------------------------------
// Engine functions.

// Called at start of gameplay.
simulated function BeginPlay()
{
	// Remember initial light type and set new one.
	Disable( 'Tick' );
	InitialBrightness = LightBrightness;
	if( bInitiallyOn )
	{
		Alpha     = 1.0;
		Direction = 1.0;
	}
	else
	{
		LightBrightness = 0; // Added by Dawn to fix Client problem
		ScaleGlow = 0; // Added by Dawn to fix Client problem
		Alpha     = 0.0;
		Direction = -1.0;
	}
	DrawType = DT_Sprite;
}

// Called whenever time passes.
simulated function Tick( float DeltaTime )
{
	if (Level.NetMode == NM_Client)
	{
		ScaleGlow = LightBrightness / InitialBrightness;
		return;
	}
	Alpha += Direction * DeltaTime / ChangeTime;
	if( Alpha > 1.0 )
	{
		Alpha = 1.0;
		Disable( 'Tick' );
		if( SavedTrigger != None )
			SavedTrigger.EndEvent();
	}
	else if( Alpha < 0.0 )
	{
		Alpha = 0.0;
		Disable( 'Tick' );
		if( SavedTrigger != none )
			SavedTrigger.EndEvent();
	}
	if( !bDelayFullOn ) // Slowly fade on or fade off
	{
		LightBrightness = Alpha * InitialBrightness;
		ScaleGlow = Alpha;
	}
	else if( (Direction>0 && Alpha!=1) || Alpha==0 )
	{
		LightBrightness = 0.0;
		ScaleGlow = 0.0;
	}
	else
	{
		LightBrightness = InitialBrightness;
		ScaleGlow = 1.0;
	}
}

function PlayRunningSounds()
{
	if( Direction == 1.0 )
	{
		PlaySound( TurnOnSound, SLOT_None );
		AmbientSound = RunningAmbientSound;
	}
	else
	{
		PlaySound( TurnOffSound, SLOT_None );
		AmbientSound = none;
	}
}

//-----------------------------------------------------------------------------
// Public states.

// Trigger turns the light on.
state() TriggerTurnsOn
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = 1.0;
		PlayRunningSounds();
		Enable( 'Tick' );
	}
}

// Trigger turns the light off.
state() TriggerTurnsOff
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = -1.0;
		PlayRunningSounds();
		Enable( 'Tick' );
	}
}

// Trigger toggles the light.
state() TriggerToggle
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction *= -1;
		PlayRunningSounds();
		Enable( 'Tick' );
	}
}

// Trigger controls the light.
state() TriggerControl
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		if( bInitiallyOn )
		{
			Direction = -1.0;
			PlayRunningSounds();
		}
		else
		{
			Direction = 1.0;
			PlayRunningSounds();
		}
		Enable( 'Tick' );
	}
	function UnTrigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		if( bInitiallyOn )
		{
			Direction = 1.0;
			PlayRunningSounds();
		}
		else
		{
			Direction = -1.0;
			PlayRunningSounds();
		}
		Enable( 'Tick' );
	}
}

state() TriggerPound {

	function Timer () {

		if (poundTime >= RemainOnTime) {

			Disable ('Timer');
		}
		poundTime += ChangeTime;
		Direction *= -1;
		PlayRunningSounds();
		SetTimer (ChangeTime, false);
	}

	function Trigger( actor Other, pawn EventInstigator )
	{

		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = 1;
		PlayRunningSounds();
		poundTime = ChangeTime;			// how much time will pass till reversal
		SetTimer (ChangeTime, false);		// wake up when it's time to reverse
		Enable   ('Timer');
		Enable   ('Tick');
	}
}

defaultproperties
{
     bStatic=False
     bHidden=False
     DrawType=DT_Sprite
     RemoteRole=ROLE_SimulatedProxy
     ScaleGlow=1.000000
     bMovable=True
}
