//=============================================================================
// DePressureZone for UT.  By Dawn Knight version 1.02 08/29/2004
//
// Same as std PressureZone, except adds an UnTrigger function and can remove
// Player's TransLocator while in Zone and return it upon leaving.
//=============================================================================
class DePressureZone extends ZoneInfo;

var() float  KillTime;					// How long to kill the player?
var() float  StartFlashScale;			// Fog values for client death sequence
var() vector StartFlashFog;
var() float  EndFlashScale;
var() Vector EndFlashFog;
var() float  DieFOV;					// Field of view when dead (interpolates)
var() float  DieDrawScale;				// Drawscale when dead
var   float  TimePassed;
var() byte   DieFatness;				// Fatness when dead
var   bool   bTriggered;				// Ensure that it doesn't update until it should
var	  bool	 bScreamed;
var() bool bNoTransLocators; // Take away Player's TransLocators in this Zone
var() string NoTransLocatorMsg; // Tell Player "No TansLocators allowed"

function BeginPlay()
{
	Super.BeginPlay();
	Disable('Tick');
	bTriggered = false;
	DieFOV = FClamp( DieFOV, 1, 170 );
	DieFatness = Clamp( DieFatness, 1, 255 );
}

function Trigger( actor Other, pawn EventInstigator )
{
	local Pawn P;

	// The DePressure zone has been triggered to kill something
	Instigator = EventInstigator;

	if ( Instigator.IsA('bot') )
	{
		// taunt the victim
		for ( P=Level.PawnList; P!=None; P=P.NextPawn )
			if( (P.Region.Zone == self) && (P.Health > 0) )
			{
				Instigator.Target = P;
				Instigator.GotoState('VictoryDance');
			}
	}
	// Engage Tick so that death may be slow and dramatic
	TimePassed = 0;
	bTriggered = true;
	bScreamed = false;
	Disable('Trigger');
	Enable('Tick');
}

function UnTrigger( actor Other, pawn EventInstigator )
{
	local Pawn P;

	Disable('Tick');
	TimePassed = 0;
	bTriggered = false;
	bScreamed = false;
	// The DePressure zone has been untriggered - so stop killing
	for ( P=Level.PawnList; P!=None; P=P.NextPawn )
	{
		if( (P.Region.Zone == self) && !P.IsA('Spectator') )
			MakeNormal(P);
	}
	Enable('Trigger');
}

function Tick( float DeltaTime )
{
	local float  		ratio, curScale;
	local vector 		curFog;
	local PlayerPawn	pPawn;
	local Pawn P;
	local bool bActive;

	if( !bTriggered )
	{
		Disable('Tick');
		return;
	}
	TimePassed += DeltaTime;
	ratio = TimePassed/KillTime;
	if( ratio > 1.0 ) ratio = 1.0;
	for ( P=Level.PawnList; P!=None; P=P.NextPawn )
	{
		// Ensure player hasn't been dispatched through other means already (suicide?)
		if( (P.Region.Zone == self) && (P.Health > 0) && !P.IsA('Spectator') )
		{
			bActive = true;
			// Fatness
			P.Fatness   = byte( 128 + Int( (Float(DieFatness)-128) * ratio ));
			P.DrawScale = 1 + (DieDrawScale-1) * ratio;
			// Maybe scream?
			if( !bScreamed && P.bIsPlayer && (Ratio > 0.2) && (FRand() < 2 * DeltaTime) )
			{
				// Scream now (from the terrible pain)
				bScreamed = true;
				P.PlaySound( P.Die, SLOT_Talk );
			}
			// Fog & Field of view
			pPawn = PlayerPawn(P);
			if( pPawn != None )
			{
				curScale = (EndFlashScale-StartFlashScale)*ratio + StartFlashScale;
				curFog   = (EndFlashFog  -StartFlashFog  )*ratio + StartFlashFog;
				pPawn.ClientFlash( curScale, 1000 * curFog );

				pPawn.SetFOVAngle( (DieFOV-pPawn.default.FOVAngle)*ratio + pPawn.default.FOVAngle);
			}
			if ( ratio == 1.0 )
			{
				Level.Game.SpecialDamageString = DamageString;
				P.health = -1000; //make sure gibs
				P.Died(Instigator, 'SpecialDamage', P.Location);
				P.bAlwaysRelevant = P.default.bAlwaysRelevant;
				MakeNormal(P);
			}
		}
	}
	if( !bActive && (TimePassed >= KillTime) )
	{
		Disable('Tick');
		Enable('Trigger');
		bTriggered = false;
	}
}

function MakeNormal(Pawn P)
{
	local PlayerPawn PPawn;
	// set the fatness back to normal
	P.Fatness = P.Default.Fatness;
	P.DrawScale = P.Default.DrawScale;
	P.AmbientSound = none;
	PPawn = PlayerPawn(P);
	if( PPawn != None )
		PPawn.SetFOVAngle( PPawn.Default.FOVAngle );
}

// When an actor enters this zone.
event ActorEntered( actor Other )
{
	local actor A;
	local vector AddVelocity, Start, Dest;
	local TransLocator TL;
	local TransLocatorTarget TT;
	local actor e;

	if( Other.IsA('TransLocatorTarget') && bNoTransLocators )
	{
		TT = TransLocatorTarget( Other );
		if( TT.Master != none )
		{
			TT.Master.bTTargetOut = false;
			TT.Master.TTarget = none;
			TT.Destroy();
			Dest = TT.Location;
			if ( TT.Physics == PHYS_None )
				Dest += vect(0,0,40);
			Start = Pawn(TT.Master.Owner).Location;
			e = Spawn(class'TranslocOutEffect',,,Dest, TT.Master.Owner.Rotation);
			e.Mesh = TT.Mesh;
			e.Animframe = TT.Animframe;
			e.Animsequence = TT.Animsequence;
			e.Velocity = 200 * Normal(Dest - Start);
		}
		else
			TT.Destroy();
		return;
	}
	if( bNoInventory && Other.IsA('Inventory') && (Other.Owner == None) )
	{
		Other.LifeSpan = 1.5;
		return;
	}
	if( Pawn(Other)!=None && Pawn(Other).bIsPlayer && !Pawn(Other).IsA('Spectator') )
	{
		Other.bAlwaysRelevant = true;
		if( ++ZonePlayerCount==1 && ZonePlayerEvent!='' )
			foreach AllActors( class 'Actor', A, ZonePlayerEvent )
				A.Trigger( Self, Pawn(Other) );
		if( bNoTransLocators )
		{
			TL = TransLocator( Pawn( Other ).FindInventoryType(class'TransLocator') );
			if (TL != None)
			{
				if( Pawn( Other ).Weapon == TL )
					Pawn( Other ).SwitchToBestWeapon();
				Pawn( Other ).DeleteInventory( TL ); // Remove it from Player's Inventory
				TL.Destroy(); // Remove it from the game
				if( TournamentPlayer( Other ) != none && NoTransLocatorMsg !="")
					Pawn( Other ).ClientMessage( NoTransLocatorMsg,	'CriticalEvent', true );
			}
			if ( Pawn( Other ).Weapon == None )
				Pawn( Other ).SwitchToBestWeapon();
		}
		return;
	}
	if ( bMoveProjectiles && (ZoneVelocity != vect(0,0,0)) )
	{
		if ( Other.Physics == PHYS_Projectile )
			Other.Velocity += ZoneVelocity;
		else if ( Other.IsA('Effects') && (Other.Physics == PHYS_None) )
		{
			Other.SetPhysics(PHYS_Projectile);
			Other.Velocity += ZoneVelocity;
		}
	}
}

// When an actor leaves this zone.
event ActorLeaving( actor Other )
{
	local actor A;
	local TransLocator TL;

	if( Other.bIsPawn )
		MakeNormal(Pawn(Other));
	if( Pawn(Other)!=None && Pawn(Other).bIsPlayer && !Pawn(Other).IsA('Spectator') )
	{
		Other.bAlwaysRelevant = Other.default.bAlwaysRelevant;
		if( --ZonePlayerCount==0 && ZonePlayerEvent!='' )
			foreach AllActors( class 'Actor', A, ZonePlayerEvent )
				A.UnTrigger( Self, Pawn(Other) );
		if ( bNoTranslocators && ( DeathMatchPlus( Level.Game ).bUseTranslocator )
				&& !PlayerPawn( Other ).IsInState( 'PlayerWaiting' ) )
		{
			// Spawn Translocator.
			if( Pawn( Other ).FindInventoryType(class'Translocator')==None )
			{
				TL = Spawn(class'Translocator');
				if( TL != none )
				{
					TL.Instigator = Pawn( Other );
					TL.BecomeItem();
					Pawn( Other ).AddInventory( TL );
					TL.GiveAmmo( Pawn( Other ) );
					TL.SetSwitchPriority( Pawn( Other ) );
					TL.WeaponSet( Pawn( Other ) );
				}
			}
		}
	}
}

defaultproperties
{
     bNoTransLocators=true
     DamageType=SpecialDamage
     DamageString="%o was depressurized by %k."
     bStatic=False
}
