//=============================================================================
// DeepSpaceTrigger for UT.  By Dawn Knight version 1.01 08/04/2004
//
// Special trigger for DM-DawnsDeepSpace.unr.
// Toggles movement of DawnsToggleMovers, enables/disables Teleporters,
// toggles visibility and on/off state of DawnsTriggerLights, and
// toggles "depressure" state of DePressureZones.

//=============================================================================
class DeepSpaceTrigger extends Triggers;

var   bool bOpened;
var() int 	GotoKeyframe;
var() float	MoveTime;
var() bool   bTriggerOnceOnly;
var() class<actor> ClassProximityType;

var() float ReTriggerDelay; //minimum time before trigger can be triggered again
var	  float TriggerTime;
var() sound TriggerSound; // for TriggerSound Effect.

var   bool bTPTimer;
var   bool bPZTimer;
var   float RemainingTPTime, RemainingPZTime;
var   pawn PZInstigator;

var() float TeleporterActiveTime;

// Trigger type.
var() enum ETriggerType
{
	TT_PlayerProximity,	// Trigger is activated by player proximity.
	TT_PawnProximity,	// Trigger is activated by any pawn's proximity
	TT_ClassProximity,	// Trigger is activated by actor of that class only
	TT_AnyProximity,    // Trigger is activated by any actor in proximity.
	TT_Shoot,		    // Trigger is activated by player shooting it.
} TriggerType;

// Is Other actor relevant to this trigger?
final function bool IsRelevant( actor Other )
{
	switch( TriggerType )
	{
		case TT_PlayerProximity:
			return Pawn(Other)!=None && Pawn(Other).bIsPlayer;
		case TT_PawnProximity:
			return Pawn(Other)!=None && ( Pawn(Other).Intelligence > BRAINS_None );
		case TT_ClassProximity:
			return ClassIsChildOf(Other.Class, ClassProximityType);
		case TT_AnyProximity:
			return true;
		case TT_Shoot:
			return ( Projectile(Other) != None );
	}
}

function PostBeginPlay()
{
	local float OriginalDelay;
	local DawnsToggleMover DTM;
	local YawTeleporter YT;

	if( event != '' )
	{
		OriginalDelay = ReTriggerDelay;
		// Adjust ReTriggerDelay to 0.5 sec > max MoveTime of any associated Movers
		foreach AllActors( class 'DawnsToggleMover', DTM, Event )
		{
			if ( !DTM.bUseTriggerMoveTime && DTM.MoveTime > ReTriggerDelay )
				ReTriggerDelay = DTM.MoveTime;
		}
		// If ReTriggerDelay was increased by MoveTime of any Movers
		if( ReTriggerDelay > OriginalDelay )
			ReTriggerDelay += 0.5; // add 1/2 sec to be safe
		if( TeleporterActiveTime < 3.0 )
			TeleporterActiveTime = 3.0;
		foreach AllActors( class 'YawTeleporter', YT, Event )
		{
			YT.bEnabled = false;
		}
	}
	Super.PostBeginPlay();
}

// Called when something touches the trigger.
function Touch( actor Other )
{
	local DawnsToggleMover DTM;
	local YawTeleporter YT;
	local DePressureZone DPZ;
	local DawnsTriggerLight DTL;

	if( IsRelevant( Other ) )
	{
		if ( ReTriggerDelay > 0 )
		{
			if ( Level.TimeSeconds - TriggerTime < ReTriggerDelay )
				return;
			TriggerTime = Level.TimeSeconds;
		}
		PlaySound(TriggerSound);
		// Call the Mover's Move function
		if( event != '' )
		{
			foreach AllActors( class 'DawnsToggleMover', DTM, Event )
			{
				if( bOpened )  // Closing
				{
					DTM.MoveKeyframe( 0, 0 );
				}
				else // Opening
				{
					DTM.MoveKeyframe( GotoKeyframe, 0 );
				}
			}
			if( bOpened )  // Closing
			{
				// Set Timer to trigger the Pressure Zone
				RemainingPZTime = ReTriggerDelay - 0.5;
				PZInstigator = pawn( Other );
				bPZTimer = true;
				SetTimer(0.5, false);
			}
			else // Opening
			{
				foreach AllActors( class 'DePressureZone', DPZ, Event )
					DPZ.UnTrigger( self, pawn( Other ) );
				foreach AllActors( class 'YawTeleporter', YT, Event )
					YT.bEnabled = true;
				foreach AllActors( class 'DawnsTriggerLight', DTL, Event )
					DTL.Trigger( self, pawn( Other ) );
				RemainingTPTime = TeleporterActiveTime;
				bTPTimer = true;
				SetTimer(0.5, false);
			}
			bOpened = !bOpened;
		}
		if( bTriggerOnceOnly )
			// Ignore future touches.
			SetCollision(False);
	}
}

function Timer()
{
	local YawTeleporter YT;
	local DePressureZone DPZ;
	local DawnsTriggerLight DTL;

	if( bTPTimer )
	{
		RemainingTPTime -= 0.5;
		if ( RemainingTPTime >= 0.5 )
			SetTimer(0.5, false);
		else
		{
			bTPTimer = false;
			if( event != '' )
			{
				foreach AllActors( class 'YawTeleporter', YT, Event )
					YT.bEnabled = false;
				foreach AllActors( class 'DawnsTriggerLight', DTL, Event )
					DTL.UnTrigger( self, none );
			}
		}
	}
	if( bPZTimer )
	{
		RemainingPZTime -= 0.5;
		if ( RemainingPZTime >= 0.5 )
			SetTimer(0.5, false);
		else
		{
			bPZTimer = false;
			if( event != '' )
				foreach AllActors( class 'DePressureZone', DPZ, Event )
					DPZ.Trigger( self, PZInstigator );
		}
	}
}

defaultproperties
{
}
