//=============================================================================
// TriggerLightD.
// A lightsource which can be triggered on or off.
// Can fade on and off, delay on and off, or flicker on
// Works in all modes of play, including NetMode
//
// Version 1.0
// By Dawn Knight
// 07/24/04
//
//=============================================================================
class TriggerLightD extends Light;

//-----------------------------------------------------------------------------
// Variables.

var() float ChangeTime;        // Time light takes to change from on to off.
var() bool  bInitiallyOn;      // Whether it's initially on.
var() bool  bDelayFullOn;      // Delay then go full-on.
var() bool  bFlickerOn;        // Flicker then go full-on.
var() float RemainOnTime;      // How long the TriggerPound effect lasts

var   float InitialBrightness; // Initial brightness.
var   float Alpha, Direction;
var   actor SavedTrigger;
var   float poundTime;

var   bool  bFoundPlayerOwner;
var   bool  bInitialDynamicLight;
var   PlayerPawn  MyPlayer;

//-----------------------------------------------------------------------------
// Engine functions.

// Called at start of gameplay.
simulated function BeginPlay()
{
	// Remember initial light type and set new one.
	Disable( 'Tick' );
	InitialBrightness = LightBrightness;
	if( bInitiallyOn )
	{
		Alpha     = 1.0;
		Direction = 1.0;
	}
	else
	{
		LightBrightness = 0; // Added by Dawn to fix Client problem
		Alpha     = 0.0;
		Direction = -1.0;
	}
	DrawType = DT_None;
}

// Called whenever time passes.
simulated function Tick( float DeltaTime )
{
	if( !bFoundPlayerOwner ) // Quick check to turn on Player's DynamicLights
	{
		if( Level.NetMode == NM_Client || Level.NetMode == NM_Standalone )
		{
			foreach AllActors( class 'PlayerPawn', MyPlayer )
			{
				if (MyPlayer.Player != None)
				{
					bFoundPlayerOwner = True;
					bInitialDynamicLight = bool(MyPlayer.ConsoleCommand
							("get ini:Engine.Engine.ViewportManager NoDynamicLights"));
					MyPlayer.ConsoleCommand
							("set ini:Engine.Engine.ViewportManager NoDynamicLights "$0);
					break;
				}
			}
			if( !bFoundPlayerOwner )
				return;
		}
		bFoundPlayerOwner = True;
	} // End of quick check to turn on DynamicLights

	Alpha += Direction * DeltaTime / ChangeTime;
	if( Alpha > 1.0 )
	{
		Alpha = 1.0;
		Disable( 'Tick' );
		if( SavedTrigger != None )
			SavedTrigger.EndEvent();
	}
	else if( Alpha < 0.0 )
	{
		Alpha = 0.0;
		Disable( 'Tick' );
		if( SavedTrigger != None )
			SavedTrigger.EndEvent();
	}
	if( !bDelayFullOn && !bFlickerOn) // Fade ON and Fade Off
		LightBrightness = Alpha * InitialBrightness;
	else if ( bDelayFullOn && !bFlickerOn) // Delay Full On
	{
		if( (Direction>0 && Alpha!=1) || Alpha==0 )
			LightBrightness = 0;
		else
			LightBrightness = InitialBrightness;
	}
	else  // Flicker On - Ignore bDelayFullOn
	{
		if( Direction<0) // Turn off immediately
		{
			LightType = LT_Steady;
			LightBrightness = Alpha * InitialBrightness;
		}
		else if( Direction>0 && Alpha<1) // Flicker
		{
			LightType = LT_Flicker;
			LightBrightness = InitialBrightness;
		}
		else  // Full On
		{
			LightType = LT_Steady;
			LightBrightness = InitialBrightness;
		}
	}
}

//-----------------------------------------------------------------------------
// Public states.

// Trigger turns the light on.
state() TriggerTurnsOn
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = 1.0;
		Enable( 'Tick' );
	}
}

// Trigger turns the light off.
state() TriggerTurnsOff
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = -1.0;
		Enable( 'Tick' );
	}
}

// Trigger toggles the light.
state() TriggerToggle
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction *= -1;
		Enable( 'Tick' );
	}
}

// Trigger controls the light.
state() TriggerControl
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		if( bInitiallyOn ) Direction = -1.0;
		else               Direction = 1.0;
		Enable( 'Tick' );
	}
	function UnTrigger( actor Other, pawn EventInstigator )
	{
		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		if( bInitiallyOn ) Direction = 1.0;
		else               Direction = -1.0;
		Enable( 'Tick' );
	}
}

state() TriggerPound {

	function Timer () {

		if (poundTime >= RemainOnTime) {

			Disable ('Timer');
		}
		poundTime += ChangeTime;
		Direction *= -1;
		SetTimer (ChangeTime, false);
	}

	function Trigger( actor Other, pawn EventInstigator )
	{

		if( SavedTrigger!=None )
			SavedTrigger.EndEvent();
		SavedTrigger = Other;
		SavedTrigger.BeginEvent();
		Direction = 1;
		poundTime = ChangeTime;			// how much time will pass till reversal
		SetTimer (ChangeTime, false);		// wake up when it's time to reverse
		Enable   ('Timer');
		Enable   ('Tick');
	}
}

defaultproperties
{
     bStatic=False
     bHidden=False
     RemoteRole=ROLE_SimulatedProxy
     bMovable=True
}
