//=============================================================================
// YawTeleporter for UT
// Version 1.01 By Dawn Knight 08/02/2004
//
// Teleports actors between different teleporters within a level
// and adds the bChangesYawAbsolute bool to create absolute yaw change.
//
// Borrows some Teleporter code from UT2004 and uses new code to fix UT bugs.
//=============================================================================
class YawTeleporter extends Teleporter;

var(Teleporter) bool     bChangesYawAbsolute; // Set yaw to Teleporter's Absolute Yaw

// Accept an actor that has teleported in.
simulated function bool Accept( actor Incoming, Actor Source )
{
	local rotator newRot, oldRot;
	local float mag;
	local vector oldDir;
	local Pawn P;

	if ( Incoming == None )
		return false;

	// Move the actor here.
	Disable('Touch');
	newRot = Incoming.Rotation;
	if (bChangesYaw)
	{
		oldRot = Incoming.Rotation;
		newRot.Yaw = Rotation.Yaw;
		if ( Source != None && !bChangesYawAbsolute)
			newRot.Yaw += (32768 + Incoming.Rotation.Yaw - Source.Rotation.Yaw);
	}
	if ( Pawn(Incoming) != None )
	{
		//tell enemies about teleport
		if ( Role == ROLE_Authority )
			for ( P=Level.PawnList; P!=None; P=P.NextPawn )
				if ( P.Enemy == Incoming )
					P.LastSeenPos = Incoming.Location;
		if ( !Pawn(Incoming).SetLocation(Location) )
		{
			log(self$" Teleport failed for "$Incoming);
			return false;
		}
		if ( (Role == ROLE_Authority) || (Level.TimeSeconds - LastFired > 0.5) )
		{
			newRot.Roll = 0;
			Pawn(Incoming).SetRotation(newRot);
			Pawn(Incoming).ViewRotation = newRot;
			Pawn(Incoming).ClientSetRotation(newRot);
			LastFired = Level.TimeSeconds;
		}
		if ( Pawn(Incoming) != None )
		{
			Pawn(Incoming).MoveTimer = -1.0;
			Pawn(Incoming).MoveTarget = self;
		}
		PlayTeleportEffect( Incoming, false);
	}
	else
	{
		if ( !Incoming.SetLocation(Location) )
		{
			Enable('Touch');
			return false;
		}
		if ( bChangesYaw )
			Incoming.SetRotation(newRot);
	}
	Enable('Touch');
	if (bChangesVelocity)
		Incoming.Velocity = TargetVelocity;
	else
	{
		if ( bChangesYaw )
		{
			if ( Incoming.Physics == PHYS_Walking )
				OldRot.Pitch = 0;
			oldDir = vector(OldRot);
			mag = Incoming.Velocity Dot oldDir;
			Incoming.Velocity = Incoming.Velocity - mag * oldDir + mag * vector(Incoming.Rotation);
		}
		if ( bReversesX )
			Incoming.Velocity.X *= -1.0;
		if ( bReversesY )
			Incoming.Velocity.Y *= -1.0;
		if ( bReversesZ )
			Incoming.Velocity.Z *= -1.0;
	}
	return true;
}

//-----------------------------------------------------------------------------
// Teleporter functions.

function Trigger( actor Other, pawn EventInstigator )
{
	local Actor A;

	bEnabled = !bEnabled;
	if ( bEnabled ) //teleport any pawns already in my radius
		foreach TouchingActors(class'Actor', A)
			PostTouch(A);
}

event Touch(Actor Other)
{
	if ( !bEnabled || (Other == None) )
		return;

	if( Other.bCanTeleport && Other.PreTeleport(Self)==false )
	{
		PendingTouch = Other.PendingTouch;
		Other.PendingTouch = self;
	}
}

// Teleporter was touched by an actor.
simulated function PostTouch( actor Other )
{
	local Teleporter D,Dest[16];
	local int i;
	local Actor A;

	// Teleport to a teleporter in this local level. If more than one, pick random.
	foreach AllActors( class 'Teleporter', D )
		if( string(D.tag)~=URL && D!=Self )
		{
			Dest[i] = D;
			i++;
			if ( i > arraycount(Dest) )
				break;
		}
	i = rand(i);
	if( Dest[i] != None )
	{
		// Teleport the actor into the other teleporter.
		if ( Other.IsA('Pawn') )
			PlayTeleportEffect(Pawn(Other), false);
		Dest[i].Accept( Other, self );
		if ( Pawn(Other) != None )
		{
			if( (Event != '') && (Other.IsA('Pawn')) )
				foreach AllActors( class 'Actor', A, Event )
					A.Trigger( Other, Other.Instigator );
		}
	}
}

defaultproperties
{
     bChangesYawAbsolute=True
     bNoDelete=False
     bStatic=False
}
